#--------------------------------------------------------------------
#--------- Definitions of classes and functions - 2D BEAM SCRIPT ----
#--------------------------------------------------------------------
##SHOULD NOT BE CHANGED BY USER 
#If this script (C) is changed, both script D (main) and A (userInput) will be affected.
numPy = 1

try:
    import numpy as np
except:
    numPy = 0
    print("Numpy not detected, fillets and other features are not supported!!")

if numPy == 1:
    import numpy as np
    
# Importing libraries
import math
from math import *
import matplotlib
matplotlib.use('AGG')
import matplotlib.pyplot as plt
import time
import os, inspect
import csv

### Preparations, do not change this as an user
date = (time.strftime("%Y%m%d")) 

### CLASS DEFINITIONS: 
#Setting up classes to structure input, do not change as user
class Project():
    pass
    
class Units():
    length = "MM"
    force = "N"
    mass = "T"
    temperature = "CELSIU"
    pass

class Options(): 
    symmetric_Beam = False #**Optional input, default value is false**
    dimension = "2D" #Only 2d is implemented
    template_3PointBending = False #**Optional input, default value is false**
    template_4PointBending = False #**Optional input, default value is false**
    includeSelfWeight = True #only true is implemented
    parametricStudy = False
    pass

class Beam():
    name = "Concrete Beam" 
    allObjects = []
    def __init__(self):
       self.allObjects.append(self)
    def add_geometry(self, _geometry):
        self.geometry = _geometry

class Plates():
    allObjects = []
    y_coord = 0 #default value
    fixedTranslation_x = True
    fixedTranslation_y = True
    def __init__(self, _typeOfPlate, _name, _x_coord):
        self.allObjects.append(self)
        self.typeOfPlate = _typeOfPlate
        self.name = _name
        self.x_coord = _x_coord
    def add_geometry(self, _geometry):
        self.geometry = _geometry
        
class Geometry():
    pass
    
class Cover():
    side = 0
    top = 0
    bot = 0
    
class Longitudinal_Reinfo():
    allObjects = []
    material = "REINFO"
    typeOfReinfo = "Long"
    y_coord = 0 #default value
    x_coord_start = 0 #default value
    x_coord_end = 0 #default value
    def __init__(self, _name, _As, _f_ym, _f_um, _e_u, _E_mod = 200000):
        self.allObjects.append(self) 
        self.name = _name
        self.As = _As #[mm^2]
        self.f_ym = _f_ym
        self.f_um = _f_um
        self.e_u = _e_u
        self.e_mod = _E_mod
        self.setname = _name + "_Set"  

class Transverse_Reinfo():
    allNames = []
    allObjects = []
    material = "REINFO"
    typeOfReinfo = "Trans"
    y_coord_bot = 0 #default value
    y_coord_top = 0 #default value
    
    def __init__(self, _name, _As, _f_ym, _f_um, _e_u, _E_mod = 200000):  
        self.allObjects.append(self) 
        self.name = _name
        self.As = _As #[mm^2]
        self.f_ym = _f_ym
        self.f_um = _f_um
        self.e_u = _e_u
        self.e_mod = _E_mod
        self.setname = _name + "_Set"   

class Section():
    allObjects = []
    def __init__(self, _spacing, _x_coord_start, _x_coord_end):
        self.spacing = _spacing
        self.x_coord_start = _x_coord_start
        self.x_coord_end = _x_coord_end
        self.allObjects.append(self)    

def createShearReinfo(_reinfo):
    bool_shearBarInMiddle = False
    allShearBars = []
    addSet( "GEOMETRYREINFOSET", _reinfo.setname )
    setCurrentShapeSet( _reinfo.setname )
    y_coord_bot = _reinfo.y_coord_bot
    y_coord_top = _reinfo.y_coord_top
    nameOfBar = "ShearBar 1"
    old_x_coord_end = None
    for obj in Section.allObjects:
        x_coord_start = obj.x_coord_start
        spacing = obj.spacing
            
        x_coord_end = obj.x_coord_end
        
        if Options.symmetric_Beam : 
            #if length of sections is longer than half beam, ignore rest
            x_coord_mid = beam.geometry.length/2
            if x_coord_end > x_coord_mid:
                x_coord_end = x_coord_mid

        nrOfBars = math.floor((x_coord_end - x_coord_start)/spacing) #rounding down
        if x_coord_start != old_x_coord_end: 
            if nameOfBar != "ShearBar 1":
                nameOfBar = "ShearBar " + str(len(allShearBars) + 1)
            createLine( nameOfBar, [ x_coord_start, y_coord_bot ], [ x_coord_start, y_coord_top ] )
            allShearBars.append(nameOfBar)
            nameOfBars = arrayCopy( [ nameOfBar ], [ spacing, 0 ], [ 0, 0 ], [ 0, 0, 0 ], nrOfBars )
            allShearBars.extend(nameOfBars)
            nameOfBar = nameOfBars[-1]
        else: 
            nameOfBars = arrayCopy( [ nameOfBar ], [ spacing, 0 ], [ 0, 0 ], [ 0, 0, 0 ], nrOfBars )
            allShearBars.extend(nameOfBars)
            nameOfBar = nameOfBars[-1]
        if Options.symmetric_Beam :
            x_coord = edgeCoordinates(nameOfBar)[0][0]
            if x_coord == beam.geometry.length/2:
                bool_shearBarInMiddle = True
                addSet( "GEOMETRYREINFOSET", "ShearBar_mid" ) 
                moveToShapeSet( [ nameOfBars[-1] ], "ShearBar_mid" )
        old_x_coord_end = x_coord_end
        Transverse_Reinfo.allNames.extend(allShearBars)  
    return bool_shearBarInMiddle
    
class Concrete(): 
    #Material Properties:
    Name = "Concrete" #Name of the material
    material = "CONCR"
    aggregate_type = "QUARTZ" #For normal weight concrete quartzite aggregates are assumed. 
    #Cement type is set to N
    material_model = "TSCR" #Total strain based cracking.
    crack_orientation = "ROTATE" #Rotating as default, will be changed to FIXED for beams without stirrups. 
    tensile_curve = "EXPONE"
    crackBandwidt_specification = "GOVIND"
    poissonsRatio_reductionModel = "DAMAGE" #Poisson's ratio reduction, damage based
    compression_curve = "PARABO"
    lateralCracking_reductionModel = "VC1993" #Reduction curve due to lateral cracking
    lateralCracking_reductionCurve_lowerBound = 0.4 #lower bound of reduction curve
    confinementModel = "VECCHI" #Stress confinement model, "NONE" is conservative
    
    
    def __init__(self, _f_ck):
        self.f_ck = _f_ck
        T31_f_ck = np.array([12,16,20,25,30,35,40,45,50,55,60,70,80,90])
        T31_f_ck_cube = np.array([15,20,25,30,37,45,50,55,60,67,75,85,95,105])
        self.f_ck_cube = np.interp(_f_ck,T31_f_ck,T31_f_ck_cube)
        self.EC2 = "C" + str(int(self.f_ck)) + "/" + str(int(self.f_ck_cube))	# Table entries
        self.fib = "C" + str(int(self.f_ck))

class Steel_Plates(): #Material model for steel plates
    #A linear-elastic behaviour is assumed
    material = "MCSTEL"
    material_model = "ISOTRO"
    density = 0 #Do not wish to take the weight of the plates into concideration
    pass

    
def concreteFromEC2():
    addMaterial( "Concrete_EC2", "CONCDC", "EN1992", [ "TOTCRK" ] )
    setParameter( "MATERIAL", "Concrete_EC2", "EC2CON/NORMAL/CLASS", concrete.EC2)
    setParameter( "MATERIAL", "Concrete_EC2", "EC2CON/NORMAL/AGGTYP", concrete.aggregate_type)
    return "Concrete_EC2"

def concreteFromFib2010():
    addMaterial( "Concrete_fib2010", "CONCDC", "MC2010", [ "TOTCRK" ] )
    setParameter( "MATERIAL", "Concrete_fib2010", "MC10CO/NORMAL/GRADE", concrete.fib)
    setParameter( "MATERIAL", "Concrete_fib2010", "MC10CO/NORMAL/AGGTYP", concrete.aggregate_type )
    return "Concrete_fib2010"

class Selfweight():
    name = "gravity"
    setname = "Gravity"

class Loads():
    allObjects = []
    setname = "Loads"
    def __init__(self, _typeOfLoad, _name, _value, _direction = "Normal"):
        self.allObjects.append(self)
        self.typeOfLoad = _typeOfLoad
        self.value = _value
        self.direction = _direction
        self.name = _name
    def attachLoadToTargets(self, _target, _beam):
        if self.typeOfLoad == "Point":
            #Target is a loadplate:
            #addSet( "GEOMETRYLOADSET", self.setname )
            createPointLoad( self.name, self.setname )
            setParameter( "GEOMETRYLOAD", self.name, "FORCE/VALUE", self.value )
            if self.direction == "X":
                setParameter( "GEOMETRYLOAD", self.name, "FORCE/DIRECT", 1 )
            elif self.direction == "Y":
                setParameter( "GEOMETRYLOAD", self.name, "FORCE/DIRECT", 2 )
            attach( "GEOMETRYLOAD", self.name, _target.name, 
            [[   _target.x_coord, _beam.geometry.height+_target.geometry.height ]] )
        elif self.typeOfLoad == "Distributed":
            #Target is an edge of a shape, ex. part top of beam:
            #Direction = normal to surface
            createLineLoad( self.name, self.setname )
            setParameter( "GEOMETRYLOAD", self.name, "FORCE/VALUE", self.value )
            attach( "GEOMETRYLOAD", self.name, _target.name, [ _target.x_coord, _target.geometry.height] )

class Interface():
    #Default values retrieved from de Putter's beamscript. 
    name = "Interface"
    Knn_tension     = 1e-9   # N/mm3 
    Knn_compression = 1e+3   # N/mm3
    Kt              = 1.0    # N/mm3 
    pass
    
class Mesh():
    meshorder = "QUADRATIC"
    meshertype = "HEXQUAD"
    pass
    
    
class Analysis():
    allObjects = []
    def __init__(self, _name, _typeOfAnalysis):
        self.name = _name
        self.typeOfAnalysis = _typeOfAnalysis
        if self.typeOfAnalysis == "Linear":
            self.command = "Structural linear static"
            self.method = "ITERAT"
            self.output = "PRIMAR"
            self.runSolver = False
        elif self.typeOfAnalysis == "Nonlinear":
            self.command = "Structural nonlinear"
            self.allConvergenceNormsHaveToBeSatisfied = False
            self.continueIfNoConvergence = True
            self.runSolver = False
            self.lineSearch = True
            self.arcLengthControl = True
        self.allObjects.append(self)
    def setConvergence(self,_Convergence):
        self.convergence = _Convergence
    def setIterationMethod(self, _Iteration):
        self.iteration = _Iteration
    def setIncrementalMethod(self, _Incrementation):
        self.incrementation = _Incrementation


class Incrementation():
    pass

class Iteration():
    firstStiffnessMatrix = "TANGEN"
    pass

class Convergence():
    useForceNorm = True    
    useEnergyNorm = True   
    useDispNorm = False 
    pass
    
class Scalefactor():
    loadFactor_plot = 1
    displacement_plot = 1
    loadFactor_CSV = 1
    displacement_CSV = 1
    pass

class LoadDispY_Graph():
    xlabel = "Displacement [mm]"
    ylabel = "Loadfactor"

class LoadDispY_CSV():
    decimalPlaces = 2

def getLoadDispYArrays(x_coord,y_coord, _analysis, scaleFactor_loadFactor, scaleFactor_displacement):
    nodeVector = findNodesCloseTo((x_coord,y_coord), Mesh.elementsize/4)
    nodeToBeUsed = nodeVector[0]
    results_table = resultsTable({"analysis": _analysis.name, "result": "Displacements",
    "components": ["TDtY"], "nodes": [nodeToBeUsed], "cases": (resultCases(_analysis.name))})

    loadFactor_vec = []
    displacement_vec = []
    for k in range(1,len(results_table)):
        caseName = results_table[k][0]
        dum = caseName.split(", ")
        dum = dum[1]
        dum = dum.split()
        loadfac = float(dum[1])
        loadFactor_vec.append(loadfac)
        displacement_vec.append(results_table[k][2])   
    # Transforming into arrays
    loadFactor_arr = np.array(loadFactor_vec)
    displacement_arr = np.array(displacement_vec)
    #Scaling arrays
    loadFactor_arr  = loadFactor_arr*scaleFactor_loadFactor
    displacement_arr = displacement_arr*scaleFactor_displacement
    return loadFactor_arr, displacement_arr

def loadDispYPlot(x_coord,y_coord, _analysis, scaleFactor_loadFactor, scaleFactor_displacement):
    loadFactor_arr, displacement_arr = getLoadDispYArrays(x_coord,y_coord, _analysis, 
    scaleFactor_loadFactor, scaleFactor_displacement)
    # Generating plot
    fig = plt.figure()
    ax1 = fig.add_subplot(1, 1, 1)
    ax1.set_xlabel(LoadDispY_Graph.xlabel)
    ax1.set_ylabel(LoadDispY_Graph.ylabel)
    if hasattr(LoadDispY_Graph, 'xLim'):
        ax1.set_xlim(LoadDispY_Graph.xLim)
    if hasattr(LoadDispY_Graph, 'yLim'):
        ax1.set_ylim(LoadDispY_Graph.yLim)
    ax1.plot(-displacement_arr, loadFactor_arr, label = "Load_deflection_graph")
    # Saving figure
    #  Creating target directory
    dirName = Project.dirPath + "/" + Project.modelName + "/Plots"
    if not os.path.exists(dirName):
        os.makedirs(dirName) 
    if Options.parametricStudy == True:
        file_name = dirName + "/LoadDispY" + "_" + str(LoadDispY_Graph.parametric)
    else: 
        file_name = dirName + "/LoadDispY"
    plt.savefig(file_name + ".png", dpi=400)
    plt.close(fig)
    return 
    
     
def getLoadDisplacementCSV(x_coord,y_coord,_analysis,scaleFactor_loadFactor, scaleFactor_displacement, 
    decimalPlaces):
        # Exporting results to CSV
        loadFactor_arr, displacement_arr = getLoadDispYArrays(x_coord,y_coord, _analysis,
        scaleFactor_loadFactor,scaleFactor_displacement)
        displacement_arr = displacement_arr*-1 #Defining downwards displacement as positive
        if Options.parametricStudy == True:
            file_name = r"loadDispY" + "_" + str(LoadDispY_CSV.parametric) + ".csv"
        else: 
            file_name = r'loadDispY.csv'
        with open(file_name, 'w', newline='') as f:
            # f.write(Project.name; Project.modelName;  Project.modelExtraInfo; Nodenr: str(_node)'\n')
            writer = csv.writer(f, delimiter = ";")
            if scaleFactor_loadFactor != 1 and scaleFactor_displacement != 1:
                f.write("Scaled Displacement [mm];Scaled Loadfactor\n")
            elif scaleFactor_loadFactor != 1:
                f.write("Displacement [mm];Scaled Loadfactor\n")
            elif scaleFactor_displacement != 1:
                f.write("Scaled Displacement [mm];Loadfactor\n")
            else: 
                f.write("Displacement [mm];LoadFactor\n")
            decimals = "%." + str(decimalPlaces)+"f"
            np.savetxt(f, np.column_stack((displacement_arr,loadFactor_arr)), decimals, delimiter = ";")
                   

def StressStrainXArrays(x_coord,y_coord, _analysis):
    nodeVector = findNodesCloseTo((x_coord,y_coord), Mesh.elementsize/4)
    nodeToBeUsed = nodeVector[0]
    strain_table = resultsTable({"analysis": _analysis.name, "result": "Total Strains", 
    "components": ["EXX"], "nodes": [nodeToBeUsed], "cases": (resultCases(_analysis.name))})
    stress_table = resultsTable({"analysis": _analysis.name, "result": "Cauchy Total Stresses", 
    "components": ["SXX"], "nodes": [nodeToBeUsed], "cases": (resultCases(_analysis.name))})
    strain_vec = []
    stress_vec = []
    for k in range(1,len(strain_table)):
        stress_vec.append(stress_table[k][-1])
        strain_vec.append(strain_table[k][-1])
    # Transforming into arrays
    stress_arr = np.array(stress_vec)
    strain_arr = np.array(strain_vec)
    return stress_arr, strain_arr

def getStressStrainXCSV(_node,_element, CSVname, _analysis, decimalPlaces):
    nodeToBeUsed = _node
    elementToBeUsed = _element
    strain_table = resultsTable({"analysis": _analysis.name, "result": "Total Strains", 
    "components": ["EXX"], "nodes": [nodeToBeUsed],"elements": [elementToBeUsed],
    "cases": (resultCases(_analysis.name))})
    stress_table = resultsTable({"analysis": _analysis.name, "result": "Cauchy Total Stresses", 
    "components": ["SXX"], "nodes": [nodeToBeUsed],"elements": [elementToBeUsed],
    "cases": (resultCases(_analysis.name))})
    strain_vec = []
    stress_vec = []
    for k in range(1,len(strain_table)):
        stress_vec.append(stress_table[k][-1])
        strain_vec.append(strain_table[k][-1])
    # Transforming into arrays
    stress_arr = np.array(stress_vec)
    strain_arr = np.array(strain_vec)
    # Exporting results to CSV
    file_name = CSVname
    with open(file_name, 'w', newline='') as f:
        print("open")
        writer = csv.writer(f, delimiter = ";")
        f.write("Strain [MPa];Stress []\n")
        decimals = "%." + str(decimalPlaces)+"f"
        np.savetxt(f, np.column_stack(( strain_arr,stress_arr)), decimals, delimiter = ";")

##Other useful BUILT-IN DIANA functions that can be used:
#nodeVector = findNodesCloseTo((x_coord,y_coord),radius)