#--------------------------------------------------------------------
#--------- Main script - 2D BEAM SCRIPT ----
#--------- DIANA/Python framework for robust NLFEA of RC beams ---
#--------------------------------------------------------------------
#This is the script to be run using DIANA
#DO NOT CHANGE THIS SCRIPT AS AN USER
#For more info and user input, open script A: userInput
#This script includes a combination of DIANA commands and python commands, 
##and can not be run in any other program than DIANA. 

### IMPORTING CLASSES; FUNCTIONS AND USERINPUT
if not 'parametric_Study' in locals():
    ##All scripts have to be saved in the same folder (Have the same working directory)
    wd = sys.path[0] #Working directory of main.py
    exec(open(wd+"\\C_classesAndFunctions.py").read())
    exec(open(wd+"\\A_UserInput.py").read())

    functionOfUserInput([0])  
        ### CREATING PROJECT
    if not hasattr(Project, 'name'):
        #**Default projectname**:
        Project.name = Project.modelName + "_" + date + "_" + Project.modelExtraInfo 
else:
    Options.parametricStudy = True
    
if not hasattr(Project, 'dirPath'):
        Project.dirPath = sys.path[0]
        
newProject( Project.dirPath + "\\" + Project.modelName + "\\" +Project.name, Project.size_in_m )
setModelAnalysisAspects( [ "STRUCT" ] )

setModelDimension(Options.dimension)
setDefaultMeshOrder( "QUADRATIC" ) #Default
setDefaultMesherType( "HEXQUAD" )#Default
setDefaultMidSideNodeLocation( "ONSHAP" )#Default

setUnit( "LENGTH", Units.length)
setUnit( "FORCE", Units.force )
setUnit( "MASS", Units.mass )
setUnit( "TEMPER", Units.temperature )

### Checking if template is used: 
if Options.template_3PointBending == True or Options.template_4PointBending == True:
    Options.symmetric_Beam = True

### CREATING THE GEOMETRY 
# -- The beam --- 
#Creating the Geometry:
addSet( "SHAPESET", "Beam" )
remove( "SHAPESET", "Shapes" )
beam.x_coord_mid = beam.geometry.length/2

#Coordinates: 
if Options.symmetric_Beam == True: 
    beam.coord =[[    0,   0 ],
                [ beam.x_coord_mid, 0 ],
                [ beam.x_coord_mid, beam.geometry.height],
                [    0, beam.geometry.height ]]
else:
    beam.coord =[[    0,   0 ],
                [ beam.geometry.length, 0 ],
                [ beam.geometry.length, beam.geometry.height],
                [    0, beam.geometry.height ]]
   

for obj in Beam.allObjects:
    createSheet(obj.name, obj.coord) 
    
# -- Plates --- 
addSet( "SHAPESET", "Plates" ) 
if Options.template_3PointBending == True: #and Symmetric_Beam:

    xCoord_sup = (beam.geometry.length - beam.geometry.lengthOfSpan)/2
    xCoord_load = beam.geometry.length/2
    
    supPlate = Plates("S", "Support plate", xCoord_sup) 
    supPlate.fixedTranslation_x = False
    loadPlate = Plates("L", "Loading plate", xCoord_load)


elif Options.template_4PointBending == True: #and Symmetric_Beam:

    xCoord_sup = (beam.geometry.length - beam.geometry.lengthOfSpan)/2
    xCoord_load = (beam.geometry.length - beam.geometry.lengthOfInnerSpan)/2
    
    supPlate = Plates("S", "Support plate", xCoord_sup) 
    supPlate.fixedTranslation_x = False
    loadPlate = Plates("L", "Loading plate", xCoord_load)

#All plate objects have same geometry: 
for obj in Plates.allObjects:
    obj.add_geometry(plateGeometry) 
        
#Coordinates: 
for obj in Plates.allObjects:

    if obj.typeOfPlate == "S":
        obj.coord = [[ obj.x_coord-(obj.geometry.length/2), 0 ],
                 [ obj.x_coord+(obj.geometry.length/2), 0 ],
                 [ obj.x_coord+(obj.geometry.length/2),  -obj.geometry.height],
                 [ obj.x_coord, -obj.geometry.height],
                 [ obj.x_coord-(obj.geometry.length/2), -obj.geometry.height ]]
                 
    elif obj.typeOfPlate == "L":
        obj.coord = [[ obj.x_coord-(obj.geometry.length/2), beam.geometry.height ],
                 [ obj.x_coord+(obj.geometry.length/2), beam.geometry.height ],
                 [ obj.x_coord+(obj.geometry.length/2), beam.geometry.height+ obj.geometry.height ],
                 [ obj.x_coord, beam.geometry.height+ obj.geometry.height ],
                 [ obj.x_coord-(obj.geometry.length/2), beam.geometry.height+ obj.geometry.height  ]]
    
    setCurrentShapeSet("Plates")
    createSheet(obj.name, obj.coord)
    
    if Options.symmetric_Beam == True and obj.x_coord == beam.x_coord_mid:
        addSet( "SHAPESET", "DividerSet" )
        createLine( "Divider", [ obj.x_coord, -10000], [obj.x_coord, 10000 ] )
        dum = cut( obj.name, [ "Divider" ], False, True )
        removeShape(dum[1])
        renameShape(dum[0], obj.name)
        remove( "SHAPESET", "DividerSet" )

# -- Loads --- 
#DEAD WEIGHT
if Options.includeSelfWeight: #only true is implemented
    selfweight = Selfweight()
    addSet( GEOMETRYLOADSET, selfweight.setname )
    createModelLoad( selfweight.name, selfweight.setname )

# #POINT LOAD: 
#Load
addSet( "GEOMETRYLOADSET", Loads.setname )
for obj in Loads.allObjects: 
    if Options.template_3PointBending == True or Options.template_4PointBending == True: 
        obj.target = loadPlate
    obj.attachLoadToTargets((obj.target), beam)

# -- Longitudinal Reinfo --- 
for obj in Longitudinal_Reinfo.allObjects:
    addSet( GEOMETRYREINFOSET, obj.setname)
    setCurrentShapeSet( obj.setname)
    if Options.symmetric_Beam == True: 
        obj.x_coord_end = beam.x_coord_mid
    pos1 = [ obj.x_coord_start, obj.y_coord] #[x,y]
    pos2 = [ obj.x_coord_end, obj.y_coord] #[x,y]
    createLine(obj.name, pos1,pos2)   

# -- Shear Reinfo --- 
for obj in Transverse_Reinfo.allObjects:
    bool_shearBarInMiddle = createShearReinfo(obj)

### Preparations before iterations
ListOfAllElements = Beam.allObjects + Plates.allObjects
ListOfAllElements_names = [obj.name for obj in ListOfAllElements]
ListOfAllPlates_names = [obj.name for obj in Plates.allObjects]
ListOfAllReinfoElements = Longitudinal_Reinfo.allObjects + Transverse_Reinfo.allObjects
longReinfo_names = [obj.name for obj in Longitudinal_Reinfo.allObjects]
transReinfo_names = Transverse_Reinfo.allNames
allReinfo_names = longReinfo_names + transReinfo_names 

### MATERIAL MODELS
# -- Concrete --- 
#Properties:
#Finding recommended properties
concreteFromEC2()
concreteFromFib2010()
if not hasattr(concrete, 'Youngs_modulus'):
    concrete.Youngs_modulus = parameter("MATERIAL","Concrete_EC2","EC2CON/NORMAL/PARAME/YOUDER") 
if not hasattr(concrete, 'poissons_ratio'):    
    concrete.poissons_ratio =  0.2 #RTD
if not hasattr(concrete, 'mass_density'):    
    concrete.mass_density  = parameter("MATERIAL","Concrete_EC2" ,"EC2CON/NORMAL/PARAME/DENDER")
if not hasattr(concrete, 'tensile_strength'):    
    concrete.tensile_strength = parameter("MATERIAL","Concrete_EC2" ,
    "EC2CON/NORMAL/PARAME/TOTCRK/TENDER") 
if not hasattr(concrete, 'compressive_strength'):    
    concrete.compressive_strength = parameter("MATERIAL","Concrete_EC2" ,
    "EC2CON/NORMAL/PARAME/TOTCRK/COMDER") 
if not hasattr(concrete, 'tensile_FractureEnergy'):    
    concrete.tensile_FractureEnergy = parameter("MATERIAL", "Concrete_fib2010",
    "MC10CO/NORMAL/PARAME/TOTCRK/GF1DER" ) #based upon fib Model Code
if not hasattr(concrete, 'compressive_FractureEnergy'):        
    concrete.compressive_FractureEnergy = 250*((concrete.f_ck)/
    (concrete.compressive_strength))*0.073*(concrete.compressive_strength)**(0.18) #RTD

#Changing from Rotating to fixed crack model if no shear reinforcement: 
if len(Transverse_Reinfo.allObjects) == 0:
    concrete.crack_orientation = "FIXED"

#Creating material: 
addMaterial( concrete.Name, concrete.material, concrete.material_model, []) 
setParameter( "MATERIAL", concrete.Name, "LINEAR/ELASTI/YOUNG", concrete.Youngs_modulus)
setParameter( "MATERIAL", concrete.Name, "LINEAR/ELASTI/POISON", concrete.poissons_ratio )
setParameter( "MATERIAL", concrete.Name, "LINEAR/MASS/DENSIT", concrete.mass_density )
setParameter( "MATERIAL", concrete.Name, "MODTYP/TOTCRK", concrete.crack_orientation)
if concrete.crack_orientation == "FIXED":
    setParameter( "MATERIAL", concrete.Name, "SHEAR/SHRCRV", "DAMAGE" )
setParameter( "MATERIAL", concrete.Name, "TENSIL/TENCRV", concrete.tensile_curve)
setParameter( "MATERIAL", concrete.Name, "TENSIL/TENSTR", concrete.tensile_strength)
setParameter( "MATERIAL", concrete.Name, "TENSIL/CBSPEC", concrete.crackBandwidt_specification )
setParameter( "MATERIAL", concrete.Name, "TENSIL/POISRE/POIRED", concrete.poissonsRatio_reductionModel)
setParameter( "MATERIAL", concrete.Name, "TENSIL/GF1", concrete.tensile_FractureEnergy)
setParameter( "MATERIAL", concrete.Name, "COMPRS/COMCRV", concrete.compression_curve)
setParameter( "MATERIAL", concrete.Name, "COMPRS/COMSTR", concrete.compressive_strength )
setParameter( "MATERIAL", concrete.Name, "COMPRS/GC", concrete.compressive_FractureEnergy )
setParameter( "MATERIAL", concrete.Name, "COMPRS/REDUCT/REDCRV", concrete.lateralCracking_reductionModel)
setParameter( "MATERIAL", concrete.Name, "COMPRS/REDUCT/REDMIN", 
concrete.lateralCracking_reductionCurve_lowerBound)
setParameter( "MATERIAL", concrete.Name, "COMPRS/CONFIN/CNFCRV", concrete.confinementModel )

#Assigning geometry and material to beam
beam_geo = beam.name
beam_shape = beam.name

addGeometry( beam_geo, "SHEET", "MEMBRA", [] )
setParameter( "GEOMET", beam_geo, "THICK", beam.geometry.width )
setParameter( "GEOMET", beam_geo, "LOCAXS", True )
setParameter( "GEOMET", beam_geo, "LOCAXS/XAXIS", [ 1, 0, 0 ] )

setElementClassType( "SHAPE", [ beam_shape ], "MEMBRA" )
assignMaterial( concrete.Name, "SHAPE", [ beam_shape ] )
assignGeometry( beam_geo, "SHAPE", [ beam_shape ] )

#STEEL FOR SUPPORTS AND LOADING PLATES
steel_mat = "Steel Plates"
steel_geo = "Steel Plates"

addMaterial( steel_mat, Steel_Plates.material, Steel_Plates.material_model, [] )
setParameter( "MATERIAL", steel_mat, "LINEAR/MASS/DENSIT", Steel_Plates.density )
setParameter( "MATERIAL", steel_mat, "LINEAR/ELASTI/YOUNG", Steel_Plates.e_mod)
setParameter( "MATERIAL", steel_mat, "LINEAR/ELASTI/POISON", Steel_Plates.poissons_ratio )

addGeometry( steel_geo, "SHEET", "MEMBRA", [] )
plateGeometry.width = beam.geometry.width  
setParameter( "GEOMET", steel_geo, "THICK", plateGeometry.width)

setElementClassType( "SHAPE",  ListOfAllPlates_names , "MEMBRA" )
isSteel = assignMaterial( steel_mat, "SHAPE", ListOfAllPlates_names )
assignGeometry( steel_geo, "SHAPE", ListOfAllPlates_names )
if isSteel:
    setShapeColor( "#cccccc", ListOfAllPlates_names )

#Integration schemes: 
addElementData( "integrationScheme")
setParameter( "DATA", "integrationScheme", "INTEGR", "HIGH" )
assignElementData( "integrationScheme", "SHAPE", ListOfAllElements_names) 
   
# -- Reinforcement --- 
for obj in ListOfAllReinfoElements:
    obj.kapsig = [ 0, obj.f_ym, obj.e_u, obj.f_um] #For use in the bilinear stress-strain diagram

for obj in ListOfAllReinfoElements: 
    addMaterial( obj.name, "REINFO", "VMISES", [] )
    setParameter( "MATERIAL", obj.name, "LINEAR/ELASTI/YOUNG", obj.e_mod )
    setParameter( "MATERIAL", obj.name, "PLASTI/YLDTYP", "KAPSIG" )
    setParameter( "MATERIAL", obj.name, "PLASTI/HARDI2/KAPSIG", [] )
    setParameter( "MATERIAL", obj.name, "PLASTI/HARDI2/KAPSIG", obj.kapsig )
    addGeometry( obj.name, "RELINE", "REBAR", [] )
    setParameter( "GEOMET", obj.name, "REIEMB/CROSSE", obj.As )
    
    setReinforcementType( "GEOMETRYREINFOSET", obj.setname, "BAR" )

    assignMaterial( obj.name, "GEOMETRYREINFOSET", obj.setname )
    assignGeometry( obj.name, "GEOMETRYREINFOSET", [ obj.setname ] )
    
    if obj.typeOfReinfo == "Trans":
        setReinforcementDiscretization( obj.allNames, "ELEMENT" )
        if bool_shearBarInMiddle: 
            #Creating a shear bear with half the area in middle of beam,
            #so it will be correct when modelling symmetric
            assignMaterial( obj.name, "GEOMETRYREINFOSET", "ShearBar_mid")
            addGeometry( obj.name + "_mid" , "RELINE", "REBAR", [] )
            assignGeometry( obj.name + "_mid", "GEOMETRYREINFOSET", [ "ShearBar_mid" ] )
            setParameter( "GEOMET", obj.name + "_mid" , "REIEMB/CROSSE", obj.As/2 )
            setContinuousInInterfaces( "GEOMETRYREINFOSET", "ShearBar_mid", False )
    else:
        setReinforcementDiscretization( [ obj.name], "ELEMENT" )    
    setContinuousInInterfaces( "GEOMETRYREINFOSET", obj.setname, False )

# -- Supports --- 
if Options.symmetric_Beam == True:
    #Symmetric support in middle of beam
    nameOfSupport = "Symmetry"
    addSet( "GEOMETRYSUPPORTSET", "Symmetric supports" )
    createLineSupport( nameOfSupport, "Symmetric supports" )
    setParameter( "GEOMETRYSUPPORT", nameOfSupport, "AXES", [ 1, 2 ] )
    setParameter( "GEOMETRYSUPPORT", nameOfSupport, "TRANSL", [ 1, 0 ] )
    setParameter( "GEOMETRYSUPPORT", nameOfSupport, "ROTATI", [ 0, 0, 1] )
    attach( "GEOMETRYSUPPORT", nameOfSupport, beam.name, [ [ beam.x_coord_mid, beam.geometry.height/2]])
    for obj in Plates.allObjects:
        if obj.x_coord == beam.x_coord_mid:
            if obj.typeOfPlate == "S":
                attach( "GEOMETRYSUPPORT", nameOfSupport, obj.name, [ [ obj.x_coord,
                beam.geometry.height - obj.geometry.height/2 ] ] )
            elif obj.typeOfPlate == "L":
                attach( "GEOMETRYSUPPORT", nameOfSupport, obj.name, [ [ obj.x_coord, 
                beam.geometry.height + obj.geometry.height/2 ] ] )

addSet( "GEOMETRYSUPPORTSET", "Supports" )    
counter = 1
for obj in Plates.allObjects:
    if obj.typeOfPlate == "S":
        x_trans = 1
        y_trans = 1
        nameOfSupport = "BC" + str(counter)
        createPointSupport( nameOfSupport, "Supports" )
        setParameter( "GEOMETRYSUPPORT", nameOfSupport, "AXES", [ 1, 2 ] )
        print(x_trans)
        if not obj.fixedTranslation_x: 
            x_trans = 0
        if not obj.fixedTranslation_y:
            y_trans = 0
        setParameter( "GEOMETRYSUPPORT", nameOfSupport, "TRANSL", [ x_trans, y_trans ] ) 
        setParameter( "GEOMETRYSUPPORT", nameOfSupport, "ROTATI", [ 0, 0, 0 ] ) 
        attach( "GEOMETRYSUPPORT", nameOfSupport, obj.name, [[ obj.x_coord, -obj.geometry.height]] )
        counter += 1
       

# -- Interface --- 
# Imprint the plates into the beam
for obj in Plates.allObjects: 
    imprintIntersection( beam.name , obj.name, True )  

#Creating the interface material: 
addMaterial( Interface.name, "INTERF", "NONLIF", [] ) #nonlinear relation
setParameter( MATERIAL, Interface.name, "LINEAR/IFTYP", "LIN2D" )
setParameter( MATERIAL, Interface.name, "LINEAR/ELAS2/DSNY", Interface.Knn_compression )
setParameter( MATERIAL, Interface.name, "LINEAR/ELAS2/DSSX", Interface.Kt )
setParameter( MATERIAL, Interface.name, "NONLIN/IFNOTE", "DIAGRM" )
setParameter( MATERIAL, Interface.name, "NONLIN/NLEL2/DUSTNY",
[ -1000, -Interface.Knn_compression*1e3,
	0,     0,
	1000,  Interface.Knn_tension*1e3       ] )

#Creating the Interface geometry:
addGeometry( Interface.name, "LINE", "STLIIF", [] )
setParameter( GEOMET, Interface.name, "LIFMEM/THICK", beam.geometry.width)

#creating the Interface
createConnection( Interface.name, "INTER", "SHAPEEDGE", "SHAPEEDGE" ) #Interface between two edges
setParameter( "GEOMETRYCONNECTION", Interface.name, "MODE", "CLOSED" )
setElementClassType( "GEOMETRYCONNECTION", Interface.name, "STLIIF" )

assignMaterial( Interface.name, "GEOMETRYCONNECTION", Interface.name )
assignGeometry( Interface.name, "GEOMETRYCONNECTION", Interface.name )
setParameter( "GEOMETRYCONNECTION", Interface.name, "FLIP", False )
for obj in Plates.allObjects:
    if obj.x_coord == beam.x_coord_mid and Options.symmetric_Beam == True:
        if obj.typeOfPlate == "S":
            attachTo( "GEOMETRYCONNECTION", Interface.name, "SOURCE", obj.name, 
            [[ obj.x_coord - obj.geometry.length/4, 0 ]] )
            attachTo( "GEOMETRYCONNECTION", Interface.name, "TARGET", beam.name,
            [[ obj.x_coord - obj.geometry.length/4, 0 ]] )
        elif obj.typeOfPlate == "L":
            attachTo( "GEOMETRYCONNECTION", Interface.name, "SOURCE", obj.name, 
            [ [ obj.x_coord - obj.geometry.length/4, beam.geometry.height ] ] ) 
            attachTo( "GEOMETRYCONNECTION", Interface.name, "TARGET", beam.name, 
            [[ obj.x_coord - obj.geometry.length/4, beam.geometry.height ]] )
    elif obj.typeOfPlate == "S":
        attachTo( "GEOMETRYCONNECTION", Interface.name, "SOURCE", obj.name, [[ obj.x_coord, 0 ]] )
        attachTo( "GEOMETRYCONNECTION", Interface.name, "TARGET", beam.name, [[ obj.x_coord, 0 ]] )
    elif obj.typeOfPlate == "L":
        attachTo( "GEOMETRYCONNECTION", Interface.name, "SOURCE", obj.name, 
        [ [ obj.x_coord, beam.geometry.height ] ] ) 
        attachTo( "GEOMETRYCONNECTION", Interface.name, "TARGET", beam.name, 
        [[ obj.x_coord, beam.geometry.height ]] )

#Adding element data to Interface
addElementData( "InterfaceData" )
setParameter( DATA, "InterfaceData", "INTEGR", "HIGH" )
assignElementData( "InterfaceData", "GEOMETRYCONNECTION", Interface.name )
  
# -- Mesh --- 
setElementSize( ListOfAllElements_names , Mesh.elementsize, -1, True )
setMesherType( ListOfAllElements_names , Mesh.meshertype )
#setMidSideNodeLocation( ListOfAllElements_names, Mesh.midSideNodeLocation )
generateMesh( [] )

# -- Linear Analysis --- 
for analysis in Analysis.allObjects:
    if analysis.typeOfAnalysis == "Linear":
    
        addAnalysis( analysis.name)
        addAnalysisCommand( analysis.name, "LINSTA", analysis.command)
        setAnalysisCommandDetail(  analysis.name, analysis.command, "SOLVE/TYPE", analysis.method)
        setAnalysisCommandDetail(  analysis.name, analysis.command, "OUTPUT(1)/SELTYP", analysis.output)

        
        if analysis.runSolver == True:
            runSolver( [ analysis.name ] )
            
            if hasattr(analysis, 'expectedFailureLoad'):
                [midNodeBeam] = findNodesCloseTo(((beam.geometry.length)/2,0))
                #Displacement at midpoint: 
                results_table = resultsTable({"analysis": analysis.name, "result": "Displacements",
                "components": ["DtY"], "nodes": [midNodeBeam], "cases": (resultCases(analysis.name))})
                dispAtExpectedLoad = analysis.expectedFailureLoad*results_table[2][2]
                print(dispAtExpectedLoad)
            

    # -- Nonlinear Analysis --- 
    # First selfweight, then point loads. 
    elif analysis.typeOfAnalysis == "Nonlinear":
    
        addAnalysis( analysis.name )
        addAnalysisCommand( analysis.name, "NONLIN", analysis.command )
        
        #Dead weight
        #All self_weight is applied in one increment
        renameAnalysisCommandDetail( analysis.name, analysis.command, "EXECUT(1)", selfweight.name)
        addAnalysisCommandDetail( analysis.name, analysis.command, "EXECUT(1)/LOAD/LOADNR" )
        setAnalysisCommandDetail( analysis.name, analysis.command, "EXECUT(1)/LOAD/LOADNR", 1) 
        
        #Point loads
        setAnalysisCommandDetail( analysis.name, analysis.command, "EXECUT/EXETYP", "LOAD" )
        renameAnalysisCommandDetail( analysis.name, analysis.command, "EXECUT(2)", "Loads" ) 
        addAnalysisCommandDetail( analysis.name, analysis.command, "EXECUT(2)/LOAD/LOADNR" )
        setAnalysisCommandDetail( analysis.name, analysis.command, "EXECUT(2)/LOAD/LOADNR", 2)
        
        for i in [1,2]: #only two phases
            setAnalysisCommandDetail( analysis.name, analysis.command, "EXECUT("+str(i)+")/ITERAT/MAXITE", 
            analysis.iteration.nrOfIterations )
            if analysis.iteration.method == "NEWTON-RAPHSON":
                setAnalysisCommandDetail( analysis.name, analysis.command, 
                "EXECUT("+str(i)+")/ITERAT/METHOD/METNAM", "NEWTON" )
                setAnalysisCommandDetail( analysis.name, analysis.command, 
                "EXECUT("+str(i)+")/ITERAT/METHOD/NEWTON/TYPNAM", analysis.iteration.typeOfMethod )
                setAnalysisCommandDetail( analysis.name, analysis.command, 
                "EXECUT("+str(i)+")/ITERAT/LINESE", analysis.lineSearch ) #Bool value

            if analysis.convergence.useForceNorm == True:
                setAnalysisCommandDetail( analysis.name, analysis.command, 
                "EXECUT("+str(i)+")/ITERAT/CONVER/FORCE", True )
                setAnalysisCommandDetail( analysis.name, analysis.command, 
                "EXECUT(1)/ITERAT/CONVER/FORCE/TOLCON", analysis.convergence.forceNorm_deadLoad)
                setAnalysisCommandDetail( analysis.name, analysis.command, 
                "EXECUT(2)/ITERAT/CONVER/FORCE/TOLCON", analysis.convergence.forceNorm )
                if analysis.continueIfNoConvergence == True:
                    setAnalysisCommandDetail( analysis.name, analysis.command, 
                    "EXECUT("+str(i)+")/ITERAT/CONVER/FORCE/NOCONV", "CONTIN" )
                else:
                    setAnalysisCommandDetail( analysis.name, analysis.command, 
                    "EXECUT("+str(i)+")/ITERAT/CONVER/FORCE/NOCONV", "TERMIN" )
            else:
                setAnalysisCommandDetail( analysis.name, analysis.command, 
                "EXECUT("+str(i)+")/ITERAT/CONVER/FORCE", False )
                
            if analysis.convergence.useEnergyNorm == True:
                setAnalysisCommandDetail( analysis.name, analysis.command, 
                "EXECUT("+str(i)+")/ITERAT/CONVER/ENERGY", True )
                setAnalysisCommandDetail( analysis.name, analysis.command, 
                "EXECUT(1)/ITERAT/CONVER/ENERGY/TOLCON", analysis.convergence.energyNorm_deadLoad)
                setAnalysisCommandDetail( analysis.name, analysis.command, 
                "EXECUT(2)/ITERAT/CONVER/ENERGY/TOLCON", analysis.convergence.energyNorm)
                if analysis.continueIfNoConvergence == True:
                    setAnalysisCommandDetail( analysis.name, analysis.command, 
                    "EXECUT("+str(i)+")/ITERAT/CONVER/ENERGY/NOCONV", "CONTIN" )
                else:
                    setAnalysisCommandDetail( analysis.name, analysis.command,
                    "EXECUT("+str(i)+")/ITERAT/CONVER/ENERGY/NOCONV", "TERMIN" )
            else:
                setAnalysisCommandDetail( analysis.name, analysis.command, 
                "EXECUT("+str(i)+")/ITERAT/CONVER/ENERGY", False )
            #OBS!!! IF dispNorm is removed, you still need to keep the line with dispNorm = False        
            if analysis.convergence.useDispNorm == True:
                setAnalysisCommandDetail( analysis.name, analysis.command, 
                "EXECUT("+str(i)+")/ITERAT/CONVER/DISPLA", True )
                setAnalysisCommandDetail( analysis.name, analysis.command, 
                "EXECUT(1)/ITERAT/CONVER/DISPLA/TOLCON", analysis.convergence.dispNorm_deadLoad)
                setAnalysisCommandDetail( analysis.name, analysis.command, 
                "EXECUT(2)/ITERAT/CONVER/DISPLA/TOLCON", analysis.convergence.dispNorm)
                if analysis.continueIfNoConvergence == True:
                    setAnalysisCommandDetail( analysis.name, analysis.command, 
                    "EXECUT("+str(i)+")/ITERAT/CONVER/DISPLA/NOCONV", "CONTIN" )
                else:
                    setAnalysisCommandDetail( analysis.name, analysis.command, 
                    "EXECUT("+str(i)+")/ITERAT/CONVER/DISPLA/NOCONV", "TERMIN" )
            else:
                 setAnalysisCommandDetail( analysis.name, analysis.command, 
                 "EXECUT("+str(i)+")/ITERAT/CONVER/DISPLA", False ) #This line have to be kept!!!
                
            if analysis.allConvergenceNormsHaveToBeSatisfied == True:
                setAnalysisCommandDetail(analysis.name, analysis.command, 
                "EXECUT("+str(i)+")/ITERAT/CONVER/SIMULT", True )
                    
        ## Point loads: 
        if analysis.incrementation.method == "ENERGY" and analysis.arcLengthControl == True:
            setAnalysisCommandDetail( analysis.name, analysis.command, 
            "EXECUT(2)/LOAD/STEPS/STEPTY", "ENERGY" ) 
            setAnalysisCommandDetail( analysis.name, analysis.command, 
            "EXECUT(2)/LOAD/STEPS/ENERGY/INISIZ", analysis.incrementation.initial_step_size ) 
            setAnalysisCommandDetail( analysis.name, analysis.command, 
            "EXECUT(2)/LOAD/STEPS/ENERGY/MAXSIZ", analysis.incrementation.max_step_size ) 
            setAnalysisCommandDetail( analysis.name, analysis.command,
            "EXECUT(2)/LOAD/STEPS/ENERGY/MINSIZ", analysis.incrementation.min_step_size ) 
            setAnalysisCommandDetail( analysis.name, analysis.command, 
            "EXECUT(2)/LOAD/STEPS/ENERGY/NSTEPS", analysis.incrementation.nrOfSteps ) 
            setAnalysisCommandDetail( analysis.name, analysis.command,
            "EXECUT(2)/LOAD/STEPS/ENERGY/ARCLEN", True )
            addAnalysisCommandDetail( analysis.name, analysis.command, 
            "EXECUT(2)/LOAD/STEPS/ENERGY/ARCLEN/REGULA/SET" )
            #Arc length control is done over whole beam.
            nodes = list( nodeIds( ELEMENTSET, beam.name) )
            setAnalysisCommandDetail( analysis.name, analysis.command, 
            "EXECUT(2)/LOAD/STEPS/ENERGY/ARCLEN/REGULA/SET(1)/NODES(1)/RNGNRS", nodes)
            setAnalysisCommandDetail( analysis.name, analysis.command, 
            "EXECUT(2)/LOAD/STEPS/ENERGY/ARCLEN/REGULA/SET(1)/DIRECT", 2 ) #y-dir
            # Energy based method has to be combined with arc length
     
        #Output:
        setAnalysisCommandDetail( analysis.name, analysis.command, "OUTPUT(1)/SELTYP", "USER" )
        addAnalysisCommandDetail( analysis.name, analysis.command, "OUTPUT(1)/USER" )

        output_commands = {
            "DISPLA TOTAL TRANSL GLOBAL" : "OUTPUT(1)/USER/DISPLA(1)/TOTAL" ,
            "FORCE REACTI TRANSL GLOBAL" : "OUTPUT(1)/USER/FORCE(1)/REACTI",
            "STRAIN TOTAL GREEN GLOBAL" : "OUTPUT(1)/USER/STRAIN(1)/TOTAL/GREEN",
            "STRAIN TOTAL GREEN PRINCI" : "OUTPUT(1)/USER/STRAIN(2)/TOTAL/GREEN/PRINCI",
            "STRAIN CRKWDT GREEN GLOBAL" : "OUTPUT(1)/USER/STRAIN(3)/CRKWDT",
            "STRAIN CRACK GREEN" : "OUTPUT(1)/USER/STRAIN(4)/CRACK/GREEN" ,
            "STRAIN CRKWDT GREEN PRINCI" : "OUTPUT(1)/USER/STRAIN(5)/CRKWDT/GREEN/PRINCI",
            "STRESS TOTAL CAUCHY GLOBAL" : "OUTPUT(1)/USER/STRESS(1)/TOTAL/CAUCHY",
            "STRESS TOTAL CAUCHY PRINCI" : "OUTPUT(1)/USER/STRESS(2)/TOTAL/CAUCHY/PRINCI"
        }

        for key, boolValue in analysis.output.items():
            if boolValue == True:
                addAnalysisCommandDetail( analysis.name, analysis.command, output_commands[key] ) 
                
        saveProject()
        
        if analysis.runSolver == True:
            runSolver( [ analysis.name] )
            
            if hasattr(LoadDispY_Graph, 'x_coord'):
                loadDispYPlot(LoadDispY_Graph.x_coord,LoadDispY_Graph.y_coord, analysis, 
                Scalefactor.loadFactor_plot, Scalefactor.displacement_plot)
            
            if hasattr(LoadDispY_CSV, 'x_coord'):
                getLoadDisplacementCSV(LoadDispY_CSV.x_coord,LoadDispY_CSV.y_coord, analysis,
                Scalefactor.loadFactor_CSV, Scalefactor.displacement_CSV, LoadDispY_CSV.decimalPlaces)
            
saveProject()
    